@@ -12,14 +12,18 @@ from django.contrib.auth.models import AnonymousUser
 from django.core.signing import BadSignature
 from django.core.signing import SignatureExpired
 from django.utils import timezone
+from django.conf import settings
+from django.contrib.auth import get_user_model
+User = get_user_model()
+from django.contrib.auth.models import AnonymousUser
 from oauth2_provider.views.generic import ProtectedResourceView
 from collections import OrderedDict
 from datetime import datetime
+import jwt
 import urllib
 import time
 
 
-
 class Authentication(object):
     """
     All the mixins of this module, other than ``Authentication`` can be used in
@@ -99,7 +103,9 @@ class SignatureAuthentication(Authentication):
             self.signer.unsign('%s:%s' % (string, signature), max_age=max_age)
         except BadSignature, SignatureExpires:
             return False
-        return True        
+
+        self.request.user = self.verify_request_user() 
+        return True      
 
     def get_signed_url(self, url, method, params, max_age):
         """
@@ -111,6 +117,17 @@ class SignatureAuthentication(Authentication):
         self._update_params(url, method, params, max_age)
         return '%s?%s' % (url, urllib.urlencode(params))
 
+    def verify_request_user(self):
+        """
+        Returns:
+            User instance, if correctly identified by the signature and
+            querystring.
+            None otherwise.
+
+        Override in the handler class to specify the logic.
+        """
+        return AnonymousUser()
+
     def _update_params(self, url, method, params, max_age):
         """
         Updates the ``params`` dictionary, with signature and max_age
@@ -163,16 +180,88 @@ class SignatureAuthentication(Authentication):
         return '%s:%s' % (parts[1], parts[2])
 
 
-class OAuthAuthentication(Authentication, ProtectedResourceView):
-    # TODO: Check if all apps/middlewares/models are installed correctly.
-
+class JWTAuthentication(Authentication):
     def is_authenticated(self):
         """
-        Sets the ``self.request.user`` to the user who carries the
-        Authorization token.
+        Retrieves the token from the ``Authorization`` header, verifies it,
+        makes sure it refers to a valid User instance, sets ``request.user``
+        accordingly.
         """
-        valid, r = self.verify_request(self.request)
-        if valid:
-            self.request.user = r.user
-            return True
-        return False
+        # Retrieve token from request header
+        token = self._get_token()
+        if not token:
+            return False
+
+        # Decode and verify
+        try:
+            payload = jwt.decode(
+                jwt=token, 
+                key=settings.SECRET_KEY, 
+                verify=True, 
+                verify_expiration=True,
+            )
+        except (jwt.DecodeError, jwt.ExpiredSignature):
+            return False
+        
+        # Get the user that the JWT authenticates
+        self.request.user = self.verify_request_user(payload)
+        if not self.request.user.id:
+            return False
+
+        return True
+
+    def verify_request_user(self, payload):
+        """
+        @param payload: Decrypted payload
+
+        Returns:
+            User instance, if correctly identified by the JWT token
+            AnonymousUser() otherwise.
+
+        This method assumes that the ``iss`` payload parameter contains the
+        User id, and sets the ``self.request.user`` to that corresponding User
+        instance.
+        Override in the handler class if the ``payload`` contains different data.
+        """
+        user = AnonymousUser()
+
+        user_id = payload.get('iss', None)
+        if user_id:
+            try:
+                user = User.objects.get(id=user_id)
+            except User.DoesNotExist:
+                pass
+            
+        return user            
+
+    def _get_token(self):
+        """
+        Returns the token from the ``Authorization: JWT <token>`` header
+        """
+        authorization = self.request.META.get('HTTP_AUTHORIZATION', '')
+        if not authorization:
+            return None
+
+        try:
+            type, token = authorization.split()
+        except ValueError:
+            return None
+
+        if type != 'JWT':
+            return None
+
+        return token
+
+
+
+
+                    
+
+
+
+
+# TODO
+# class ApiKeyAuthentication?
+# Or something similar that actually signs requests. Study the Amazon AWS
+# authentication scheme.
+
